@@ -87,32 +87,17 @@ gkbd_indicator_set_tooltips (GkbdIndicator * gki, const char *str);
 void
 gkbd_indicator_set_tooltips (GkbdIndicator * gki, const char *str)
 {
-	GtkTooltips *tooltips;
+	g_assert (str == NULL || g_utf8_validate (str, -1, NULL));
 
-	if (str == NULL)
-		return;
-	tooltips = gtk_tooltips_new ();
-	g_object_ref (G_OBJECT (tooltips));
-	gtk_object_sink (GTK_OBJECT (tooltips));
-	g_object_set_data_full (G_OBJECT (gki), "tooltips",
-				tooltips, (GDestroyNotify) g_object_unref);
-	gtk_tooltips_set_tip (tooltips, GTK_WIDGET (gki), str, NULL);
+	gtk_widget_set_tooltip_text (GTK_WIDGET (gki), str);
 
 	if (gki->priv->set_parent_tooltips) {
 		GtkWidget *parent =
 		    gtk_widget_get_parent (GTK_WIDGET (gki));
-		if (parent != NULL) {
-			gtk_tooltips_set_tip (tooltips,
-					      GTK_WIDGET (parent), str,
-					      NULL);
-			g_object_ref (G_OBJECT (tooltips));
-			g_object_set_data_full (G_OBJECT (parent),
-						"gnome-kbd-indicator.tooltips",
-						tooltips, (GDestroyNotify)
-						g_object_unref);
+		if (parent) {
+			gtk_widget_set_tooltip_text (parent, str);
 		}
 	}
-	gtk_tooltips_enable (tooltips);
 }
 
 void
@@ -138,8 +123,7 @@ gkbd_indicator_fill (GkbdIndicator * gki)
 		GtkWidget *page, *decorated_page = NULL;
 		gchar *full_group_name =
 		    (grp <
-		     g_strv_length (globals.
-				    full_group_names)) ?
+		     g_strv_length (globals.full_group_names)) ?
 		    globals.full_group_names[grp] : "?";
 		page = gkbd_indicator_prepare_drawing (gki, grp);
 
@@ -200,6 +184,12 @@ flag_exposed (GtkWidget * flag, GdkEventExpose * event, GdkPixbuf * image)
 	/* Image width and height */
 	int iw = gdk_pixbuf_get_width (image);
 	int ih = gdk_pixbuf_get_height (image);
+	gboolean scaling_needed =
+	    !((flag->allocation.width == iw &&
+	       flag->allocation.height <= ih) ||
+	      (flag->allocation.width >= iw &&
+	       flag->allocation.height == ih));
+
 	/* widget-to-image scales, X and Y */
 	double xwiratio = 1.0 * flag->allocation.width / iw;
 	double ywiratio = 1.0 * flag->allocation.height / ih;
@@ -213,15 +203,20 @@ flag_exposed (GtkWidget * flag, GdkEventExpose * event, GdkPixbuf * image)
 	int ox = (flag->allocation.width - sw) >> 1;
 	int oy = (flag->allocation.height - sh) >> 1;
 
-	GdkPixbuf *scaled = gdk_pixbuf_scale_simple (image, sw, sh,
-						     GDK_INTERP_HYPER);
+	GdkPixbuf *scaled =
+	    scaling_needed ? gdk_pixbuf_scale_simple (image, sw, sh,
+						      GDK_INTERP_HYPER) :
+	    image;
 
 	gdk_draw_pixbuf (GDK_DRAWABLE (flag->window),
 			 NULL,
 			 scaled,
 			 0, 0,
-			 ox, oy, sw, sh, GDK_RGB_DITHER_NORMAL, 0, 0);
-	g_object_unref (G_OBJECT (scaled));
+			 ox, oy, sw, sh,
+			 scaling_needed ? GDK_RGB_DITHER_NORMAL :
+			 GDK_RGB_DITHER_NONE, 0, 0);
+	if (scaling_needed)
+		g_object_unref (G_OBJECT (scaled));
 }
 
 static GtkWidget *
@@ -233,14 +228,16 @@ gkbd_indicator_prepare_drawing (GkbdIndicator * gki, int group)
 
 	pimage = g_slist_nth_data (globals.ind_cfg.images, group);
 	ebox = gtk_event_box_new ();
+	gtk_event_box_set_visible_window (GTK_EVENT_BOX (ebox), FALSE);
 	if (globals.ind_cfg.show_flags) {
 		GtkWidget *flag;
 		if (pimage == NULL)
 			return NULL;
 		image = GDK_PIXBUF (pimage);
 		flag = gtk_drawing_area_new ();
-		g_signal_connect (G_OBJECT (flag),
-				  "expose_event",
+		gtk_widget_add_events (GTK_WIDGET (flag),
+				       GDK_BUTTON_PRESS_MASK);
+		g_signal_connect (G_OBJECT (flag), "expose_event",
 				  G_CALLBACK (flag_exposed), image);
 		gtk_container_add (GTK_CONTAINER (ebox), flag);
 	} else {
@@ -341,6 +338,7 @@ gkbd_indicator_prepare_drawing (GkbdIndicator * gki, int group)
 
 		gtk_container_set_border_width (GTK_CONTAINER (align), 2);
 	}
+
 	g_signal_connect (G_OBJECT (ebox),
 			  "button_press_event",
 			  G_CALLBACK (gkbd_indicator_button_pressed), gki);
@@ -628,7 +626,7 @@ gkbd_indicator_init (GkbdIndicator * gki)
 		return;
 	}
 
-	gkbd_indicator_set_tooltips (gki, "");
+	gkbd_indicator_set_tooltips (gki, NULL);
 
 	gkbd_indicator_fill (gki);
 	gkbd_indicator_set_current_page (gki);
@@ -730,11 +728,6 @@ gkbd_indicator_global_init (void)
 		return;
 	}
 
-	globals.registry =
-	    xkl_config_registry_get_instance (globals.engine);
-
-	xkl_config_registry_load (globals.registry);
-
 	gconf_client = gconf_client_get_default ();
 
 	g_signal_connect (globals.engine, "X-state-changed",
@@ -758,8 +751,15 @@ gkbd_indicator_global_init (void)
 
 	gkbd_desktop_config_load_from_gconf (&globals.cfg);
 	gkbd_desktop_config_activate (&globals.cfg);
+
+	globals.registry =
+	    xkl_config_registry_get_instance (globals.engine);
+	xkl_config_registry_load (globals.registry,
+				  globals.cfg.load_extra_items);
+
 	gkbd_keyboard_config_load_from_x_current (&globals.kbd_cfg,
 						  xklrec);
+
 	gkbd_indicator_config_load_from_gconf (&globals.ind_cfg);
 	gkbd_indicator_config_update_images (&globals.ind_cfg,
 					     &globals.kbd_cfg);
